import 'package:dio/dio.dart';
import 'package:rest_api_helper/types/request_types.dart';
import 'package:rest_api_helper/utils/response.dart';

class RestApiHelper {
  late Dio _dio;

  RestApiHelper({
    required String baseUrl,
    Map<String, String>? headers,
    ResponseType? responseType,
  }) {
    _dio = Dio(BaseOptions(
      baseUrl: baseUrl,
      headers: headers,
      responseType: responseType,
    ));
  }
  Future<dynamic> send({
    required String path,
    required RequestType requestType,
    Map<String, dynamic>? data,
    Map<String, dynamic>? queryParameters,
    Map<String, String>? headers,
    void Function(int, int)? onSendProgress,
    void Function(int, int)? onRecieveProgress,
  }) async {
    dynamic request;
    dynamic response;
    switch (requestType) {
      case RequestType.post:
        try {
          request = await _dio.post(
            path,
            data: data,
            options: headers != null ? Options(headers: headers) : null,
            queryParameters: queryParameters,
            onSendProgress: onSendProgress,
            onReceiveProgress: onRecieveProgress,
          );
          response = responseCheck(
            response: request.data,
          );
        } catch (e) {
          if (e is DioError) {
            if (e.response!.statusCode != null) {
              response = responseCheck(
                response: e.response as Response,
              );
            }
          }
        }
        return response;
      case RequestType.get:
        try {
          request = await _dio.get(
            path,
            options: headers != null ? Options(headers: headers) : null,
            queryParameters: queryParameters,
            onReceiveProgress: onRecieveProgress,
          );
          response = responseCheck(
            response: request.data,
          );
        } catch (e) {
          if (e is DioError) {
            if (e.response!.statusCode != null) {
              response = responseCheck(
                response: e.response as Response,
              );
            }
          }
        }

        return response;

      case RequestType.put:
        try {
          request = await _dio.put(
            path,
            data: data,
            options: headers != null ? Options(headers: headers) : null,
            onSendProgress: onSendProgress,
            queryParameters: queryParameters,
            onReceiveProgress: onRecieveProgress,
          );
          response = responseCheck(
            response: request.data,
          );
        } catch (e) {
          if (e is DioError) {
            if (e.response!.statusCode != null) {
              response = responseCheck(
                response: e.response as Response,
              );
            }
          }
        }
        return response;

      case RequestType.delete:
        try {
          request = await _dio.delete(
            path,
            options: headers != null ? Options(headers: headers) : null,
            data: data,
            queryParameters: queryParameters,
          );
          response = responseCheck(
            response: request.data,
          );
        } catch (e) {
          if (e is DioError) {
            if (e.response!.statusCode != null) {
              response = responseCheck(
                response: e.response as Response,
              );
            }
          }
        }
        return response;
    }
  }
}
